#!/bin/sh

if [ -z ${T2C_ROOT} ]
then
        echo "Error: T2C_ROOT is not defined."
		echo "The environment variable T2C_ROOT should contain a path to the main directory of T2C Framework."
		exit 1
fi

# Get the directory where this script resides.
WORK_DIR=$(cd `dirname $0` && pwd) 
T2C_SUITE_ROOT=${WORK_DIR}
export T2C_SUITE_ROOT

#------------------------------------
cd ${T2C_SUITE_ROOT}

if [ ! -x ${T2C_ROOT}/t2c/bin/t2c ]
then
        printf "t2c executable is not found, aborting...\n"
        exit 1
fi 

export CC_SPECIAL_FLAGS=" "
echo $CC_SPECIAL_FLAGS > $T2C_SUITE_ROOT/cc_special_flags

cd ${T2C_SUITE_ROOT}
export PATH=${T2C_ROOT}/t2c/bin:$PATH

# Build all test suites
for TEST_SUITE in *-t2c
do
    if [ -d ${TEST_SUITE} ]
    then
        echo   "----------------------	"
        printf "Building $TEST_SUITE\n"

        # Generate & build tests and test data for ${TEST_SUITE}
        cd ${T2C_SUITE_ROOT}
        if [ -d ${TEST_SUITE}/testdata/testdata_src ]
        then
            cd ${T2C_SUITE_ROOT}/${TEST_SUITE}/testdata/testdata_src
            make
            if [ $? -ne 0 ]
            then
                printf "Failed to build test data, aborting...\n"
                exit 1
            fi
        fi

        cd ${T2C_SUITE_ROOT}/${TEST_SUITE}
        chmod a+x gen_tests clean
        ./gen_tests

        if [ $? -ne 0 ]
        then
            printf "Failed to generate tests for ${TEST_SUITE}, aborting...\n"
            exit 1
        fi 

        cd ${T2C_SUITE_ROOT}/${TEST_SUITE}/tests
        make
        if [ $? -ne 0 ]
        then
            printf "Failed to build tests for ${TEST_SUITE}, aborting...\n"
            exit 1
        fi

        cd ${T2C_SUITE_ROOT}
    fi 
done

# Set permissions
chmod -R a+w ${T2C_SUITE_ROOT}
if [ $? -ne 0 ]
then
        printf "Failed to set permissions. Perhaps you should use root account.\n"
        exit 1
fi

# Done
echo
echo Build completed successfully

