#!/bin/sh

# This script builds the T2C code generator and uses it
# to create the C-code of the tests without building them.

if [ -z ${T2C_ROOT} ]
then
        echo "Error: T2C_ROOT is not defined."
		echo "The environment variable T2C_ROOT should contain a path to the main directory of T2C Framework."
		exit 1
fi

# Get the directory where this script resides.
WORK_DIR=$(cd `dirname $0` && pwd) 
T2C_SUITE_ROOT=${WORK_DIR}
export T2C_SUITE_ROOT

#------------------------------------
cd ${T2C_SUITE_ROOT}

export CC_SPECIAL_FLAGS=" "
echo $CC_SPECIAL_FLAGS > $T2C_SUITE_ROOT/cc_special_flags

cd ${T2C_SUITE_ROOT}
export PATH=${T2C_ROOT}/t2c/bin:$PATH

# Remove the old TET scenario file.
rm -f ${T2C_SUITE_ROOT}/tet_scen

# Build all test suites
for TEST_SUITE in *-t2c
do
    if [ -d ${TEST_SUITE} ]
    then
        echo   "----------------------"
        printf "Generating C code for $TEST_SUITE\n"

        # Generate tests for ${TEST_SUITE}
        cd ${T2C_SUITE_ROOT}/${TEST_SUITE}
        chmod a+x gen_tests clean
        ./gen_tests

        if [ $? -ne 0 ]
        then
            printf "Failed to generate C code for ${TEST_SUITE}, aborting...\n"
            exit 1
        fi 

        cd ${T2C_SUITE_ROOT}
    fi 
done

# Done
echo
echo Code generation completed successfully

