#!/bin/sh

# Get the directory where this script resides.
WORK_DIR=$(cd `dirname $0` && pwd)

# Clean first, build later
${WORK_DIR}/clean.sh

INSTALL_RDF=${WORK_DIR}/src/install.rdf
if [ ! -f ${INSTALL_RDF} ]
then
    echo "${INSTALL_RDF} is not found."
    exit 1
fi

# Determine version number
VER_LINE=`cat ${INSTALL_RDF} | grep "em:version"`
VER_LINE=`echo ${VER_LINE} | sed 's/(^\s*)|(\s*$)//g'`
VERSION=`echo ${VER_LINE} | sed 's/<[^>]*>//g'`

XPI_NAME=reqmarkup-${VERSION}.xpi

echo "Creating ${XPI_NAME}..."

cd ${WORK_DIR}
mkdir -p xpi
cp -r src/* xpi/

# Pack inner "jar"
cd xpi/chrome
zip -rqmT reqmarkup.jar * 

if [ $? -ne 0 ] 
then
    echo "Unable to build T2C ReqMarkup: the inner jar-file is not prepared properly."
    exit 1
fi

# Pack outer "xpi"
cd ${WORK_DIR}/xpi
zip -rqmT ${XPI_NAME} * 

if [ $? -ne 0 ] 
then
	echo "Unable to build T2C ReqMarkup: failed to create xpi file."
	exit 1 
fi

# Done
echo "${XPI_NAME} is created successfully."
