var activeEditor;
var defineList;//list of defines from document, sorted by name
var stringsBox;
var currentDocument;
var defineListBox;

function DefinesDlgStartup()
{
	activeEditor = GetCurrentEditor();
	currentDocument = activeEditor.document;
	
	stringsBox = window.document.getElementById("stringsBox");
	defineListBox = window.document.getElementById("defineListBox");
	defineList = GetDefineList(currentDocument);
	showDefineList();
}

//called when selection in defineListBox changed
function itemSelected()
{
	//window.alert("ItemSelected");
	var selectedIndex = defineListBox.selectedIndex;
	if(selectedIndex < 0)
	{
		//deselection
		document.getElementById("cmd_edit_define").setAttribute("disabled", "true");
		document.getElementById("buttonEdit").setAttribute("disabled", "true");
		document.getElementById("cmd_remove_define").setAttribute("disabled", "true");
		stringsBox.value = "";
		return;
	
	}
	document.getElementById("cmd_edit_define").removeAttribute("disabled");
	document.getElementById("buttonEdit").removeAttribute("disabled");
	document.getElementById("cmd_remove_define").removeAttribute("disabled");
	
	var define = defineList[selectedIndex];
	
	if(!define.strings.length)
	{
		stringsBox.value = "(empty define extension)";
		return;
	}
	var define_strings = split_define_str(define.strings);
	stringsBox.value = define_strings.join('\n');
}
//called when execute command 'cmd_add_define'
function onAdd()
{
	var define = showDefinePropertiesDlg("", "");
	if(define == null)return;//user cancel operation
	if(!addDefine(define.name, define.value)) return;//error while adding define to document
	//reload defineList(because ordering may disturbed)
	defineList = GetDefineList(currentDocument);
	var select_index;
	for(select_index in defineList)
	{
		if(defineList[select_index].symb == define.name)break;
	}
	showDefineList(select_index);
}
//called when execute command 'cmd_edit_define'
function onEdit()
{
	var index = defineListBox.selectedIndex;
	if(index < 0) return;//no selection
	
	var define = showDefinePropertiesDlg(defineList[index].symb, defineList[index].strings);
	if(define == null) return;//user cancel operation
	editDefine(defineList[index].c, define.name, define.value);
	//reload defineList(because ordering may disturbed)
	defineList = GetDefineList(currentDocument);
	var select_index;
	for(select_index in defineList)
	{
		if(defineList[select_index].symb == define.name)break;
	}
	showDefineList(select_index);
}
//called when execute command 'cmd_remove_define'
function onRemove()
{
	var index = defineListBox.selectedIndex;
	if(index < 0) return;//no selection
	
	removedElem = defineList[index].c;
	//ordering of others element will not change, so adjusting listbox
	defineListBox.removeItemAt(index);//remove element from listbox
	defineList.splice(index, 1);//remove define from define list
	removeDefine(removedElem);//remove define-element from document
	selectDefine(index);//change selection
}

//****************Additional functions**********************
function GetDefineList(document)
{
	var result = new Array();
	try
	{
		var treeWalker = document.createTreeWalker(document.body,
			NodeFilter.SHOW_ELEMENT,
			{ acceptNode: function(node){return (node.className == "define") ? NodeFilter.FILTER_ACCEPT : NodeFilter.FILTER_SKIP;} },
			false);
		var c;
		while(c = treeWalker.nextNode())
		{
			var define = new DefineInfo(c);
			define.c = c;
			result.push(define);
		}
	}
	catch(error)
	{
		window.alert("GetDefineList: " + error.message);
	}
	result.sort(function(a,b){return a.symb.localeCompare(b.symb);});
	return result;
}
//select define at index in the listbox, adjusting visibility; if index is too large, select last element; if index is -1 - perfoms deselection
function selectDefine(index)
{
	if(index >= defineListBox.getRowCount()) index = defineListBox.getRowCount() - 1;
	if(index >= 0) defineListBox.ensureIndexIsVisible(index);
	defineListBox.selectedIndex = index;
}

//Show define list with define selected at index 'select_index'; if select_index is not specified, it assumes 0.
function showDefineList(select_index)
{
	if(!select_index) select_index = 0;//default value
	ClearListbox(defineListBox);
	
	for(var i in defineList)
	{
		defineListBox.appendItem(defineList[i].symb);
	}
	selectDefine(select_index);
}
//Show dialog for modify define properties(for add define or edit).
//Return new define-object, containing these properties, or null, if operation was canceled.
function showDefinePropertiesDlg(define_name, define_value)
{
	var define = new Object();	
	define.name = define_name;
	define.value = define_value;

	var defineNames = new Array();
	for(var i in defineList)
	{
		if(defineList[i].symb != define_name)
			defineNames.push(defineList[i].symb);
	}

	try
	{
		var dialog = window.openDialog("chrome://reqmarkup/content/EdDefineProperties.xul",
			    "_blank", 
			    "chrome,close,titlebar,resizable, modal",
				define,
				defineNames
			);
		dialog.focus();
	}
	catch(error)	
	{
		window.alert("showDefinePropertiesDlg: " + error.message);
	}
	
	if((define.name == define_name) && (define.value == define_value)) return null;
	return define;
}

//Add new define to the document
function addDefine(define_name, define_value)
{
	//create new element
	var define_element = activeEditor.createElementWithDefaults("anchor");
	//attributes in reverse order
	define_element.setAttribute("title", define_value);
	define_element.setAttribute("name", define_name);
	define_element.className = "define";
	//determine position for insertion
	var insert_point;
	if(defineList.length == 0)
	{
		//there is no defines in the document yet
		if(!currentDocument.body)
		{
			window.alert("Cannot find tag <body> for insert first define. Please, do this manually.");
			return false;
		}
		insert_point = currentDocument.body.firstChild;
		//look for node, which is DOM-element
		while(insert_point && insert_point.nodeType != 1) insert_point = insert_point.nextSibling;
		if(!insert_point)
		{
			window.alert("<body> hasn't any child, which is DOM-element. So cannot insert define. Please, do this manually.");
			return false;
		}
	}
	else
	{
		//insert new "define" before the first "define" in list(simply)
		insert_point = defineList[0].c;
	}
	//add define-element to document
	activeEditor.selectElement(insert_point);
	activeEditor.insertElementAtSelection(define_element, false);
	return true;
}

//Change attributes of existing define to new ones
function editDefine(old_elem, define_name, define_value)
{
	//create new element
	var define_element = activeEditor.createElementWithDefaults("anchor");
	//attributes in reverse order
	define_element.setAttribute("title", define_value);
	define_element.setAttribute("name", define_name);
	define_element.className = "define";
	//replace old element with new one
	activeEditor.selectElement(old_elem);
	activeEditor.insertElementAtSelection(define_element, true);
}

//Remove given define from the document
function removeDefine(define_elem)
{
	activeEditor.selectElement(define_elem);
	try
	{
		//this operation returns error, but really all OK
		activeEditor.insertHTML("");
	}
	catch(error){};
}